# frozen_string_literal: true

module Swamp
  module Action
    module Parameters
      def self.included(base)
        base.extend ClassMethods
        base.expose :validation_result
      end

      private

      attr_reader :input, :validation_result

      def prepare(params)
        super
        return true if self.class.param_class.nil?

        @validation_result = self.class.param_class.call(params)
        @input = validation_result.output
      end

      def method_missing(method_name, *)
        input.fetch(method_name.to_sym) { super }
      end

      def respond_to_missing?(method_name, _include_all)
        input.key?(method_name.to_sym) ? true : super
      end

      module ClassMethods
        attr_reader :param_class

        def params(&block)
          @param_class = Dry::Validation.Form do
            instance_exec(&block)
          end
        end
      end
    end
  end
end
