# frozen_string_literal: true

module Swamp
  module Controller
    def self.included(base)
      base.extend ClassMethods
    end

    def action!(klass, **kwargs)
      result = klass.new.call(params.to_unsafe_h.merge(request: request, **kwargs))

      setup_exposures!(klass, result)

      if result.render?
        render_result(result, klass)
      elsif result.redirect?
        redirect_result(result, klass)
      end
    end

    private

    def render_result(result, klass)
      setup_flash_now(result, klass)
      render result.template
    end

    def redirect_result(result, klass)
      setup_flash(result, klass)
      redirect_to result.redirection_path, result.redirection_options
    end

    def setup_exposures!(klass, result)
      klass.exposures.each do |key, value|
        instance_variable_set(value, result.exposures[key])
      end
    end

    def setup_flash_now(result, klass)
      return unless klass.exposures.key?(:flash)

      result.flash.each do |type, message|
        flash.now[type] = message
      end
    end

    def setup_flash(result, klass)
      return unless klass.exposures.key?(:flash)

      result.flash.each do |type, message|
        flash[type] = message
      end
    end

    module ClassMethods
      def action!(name, klass, *args)
        define_method name do
          input = Hash[args.map { |k| [k, send(k)] }]
          action! klass, **input
        end
      end
    end
  end
end
